package com.jz.dmp.modules.controller.dataOperation;

import com.jz.common.annotation.SysOperationLogAnnotation;
import com.jz.common.constant.CommConstant;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.page.PageInfoResponse;
import com.jz.dmp.modules.controller.DataIntegration.bean.ConflictCheckReq;
import com.jz.dmp.modules.controller.DataIntegration.bean.RealTimeSyncListDto;
import com.jz.dmp.modules.controller.DataIntegration.bean.RealTimeSyncListReq;
import com.jz.dmp.modules.controller.dataOperation.bean.DataDevTaskListDto;
import com.jz.dmp.modules.controller.dataOperation.bean.DataDevTaskListReq;
import com.jz.dmp.modules.service.DmpDevelopTaskService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * 数据运维-数据开发任务
 *
 * @author Bellamy
 * @since 2021-01-13 10:56:18
 */
@RestController
@RequestMapping("/dataDevTask")
@Api(tags = "数据运维-数据开发任务")
public class DmpDevTaskController {

    @Autowired
    private DmpDevelopTaskService dmpDevelopTaskService;

    /**
     * 列表分页查询
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @ApiOperation(value = "列表分页查询", notes = "列表分页查询")
    @PostMapping(value = "/listPage")
    public PageInfoResponse<DataDevTaskListDto> getDevTaskListPage(@RequestBody @Validated DataDevTaskListReq req, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<DataDevTaskListDto> pageInfo = new PageInfoResponse<DataDevTaskListDto>();
        if (StringUtils.isEmpty(req.getProjectId())) {
            pageInfo.setMessage("项目id不能为空！");
            pageInfo.setCode(ResultCode.PARAMS_ERROR);
            return pageInfo;
        }
        try {
            pageInfo = dmpDevelopTaskService.queryDevTaskListPage(req);
        } catch (Exception e) {
            pageInfo.setMessage("查询失败");
            pageInfo.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return pageInfo;
    }

    /**
     * 执行任务
     *
     * @return
     * @author Bellamy
     */
    @ApiOperation(value = "执行", notes = "执行")
    @GetMapping(value = "/runTask")
    @ApiImplicitParam(name = "taskId", value = "任务id")
    @SysOperationLogAnnotation(actionType = CommConstant.OPT_TYPE_RUN, descrption = "任务执行", optModular = CommConstant.MODULAR_OPTCENTER_DATADEV, isProjOpt = true)
    public JsonResult runTaskByTaskId(@RequestParam(value = "taskId") String taskId) throws Exception {
        if (StringUtils.isEmpty(taskId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "任务id不能为空！");
        }
        JsonResult list = dmpDevelopTaskService.runTaskByTaskId(taskId);
        return list;
    }

    /**
     * 运维大屏--获取任务状态
     *
     * @return
     * @author Bellamy
     * @since 2021-02-22
     */
    @ApiOperation(value = "运维大屏--重点关注任务状态", notes = "重点关注")
    @GetMapping(value = "/getTaskStatus")
    public JsonResult getTaskStatus(@RequestParam String projectId) throws Exception {
        if (StringUtils.isEmpty(projectId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "projectId不能为空！");
        }
        JsonResult result = new JsonResult();
        try {
            result = dmpDevelopTaskService.getTaskStatus(projectId);
        } catch (Exception e) {
            result.setMessage("failed!");
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 运维大屏--任务完成情况
     *
     * @return
     * @author Bellamy
     * @since 2021-03-08
     */
    @ApiOperation(value = "运维大屏--任务完成情况", notes = "运维大屏--任务完成情况")
    @GetMapping(value = "/getTaskStatusByHours")
    @ApiImplicitParams({@ApiImplicitParam(name = "projectId", value = "projectId", required = true),
            @ApiImplicitParam(name = "taskStatus", value = "50:运行成功，70：未运行", required = true)})
    public JsonResult getTaskStatusByHours(@RequestParam String projectId, @RequestParam String taskStatus) throws Exception {
        if (StringUtils.isEmpty(projectId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "projectId不能为空！");
        }
        JsonResult result = new JsonResult();
        try {
            result = dmpDevelopTaskService.getTaskStatusByHours(projectId, taskStatus);
        } catch (Exception e) {
            result.setMessage("failed");
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 获取 SLA
     *
     * @return
     * @author Bellamy
     * @since 2021-03-08
     */
    @ApiOperation(value = "获取 SLA", notes = "获取 SLA")
    @GetMapping(value = "/getSlaInfo")
    @ApiImplicitParams({@ApiImplicitParam(name = "scheduleId", value = "scheduleId", required = true),
            @ApiImplicitParam(name = "projectId", value = "projectId", required = true)})
    public JsonResult getSlaInfo(@RequestParam String scheduleId, @RequestParam String projectId) throws Exception {
        if (StringUtils.isEmpty(scheduleId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "scheduleId不能为空！");
        }
        if (StringUtils.isEmpty(projectId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "projectId不能为空！");
        }
        JsonResult result = new JsonResult();
        try {
            result = dmpDevelopTaskService.getSlaInfo(scheduleId,projectId);
        } catch (Exception e) {
            result.setMessage("failed");
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 获取 flowId/任务名称
     *
     * @return
     * @author Bellamy
     * @since 2021-03-08
     */
    @ApiOperation(value = "设置SLA--获取flowId/任务名称", notes = "获取flowId/任务名称")
    @GetMapping(value = "/getTaskFlowId")
    @ApiImplicitParam(name = "projectId", value = "projectId", required = true)
    public JsonResult getTaskFlowId(@RequestParam String projectId) throws Exception {
        if (StringUtils.isEmpty(projectId)) {
            return new JsonResult(ResultCode.PARAMS_ERROR, "projectId不能为空！");
        }
        JsonResult result = new JsonResult();
        try {
            result = dmpDevelopTaskService.getTaskFlowId(projectId);
        } catch (Exception e) {
            result.setMessage("failed");
            result.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return result;
    }
}
