package com.jz.dmp.modules.controller.dataService;

import com.jz.common.annotation.SysOperationLogAnnotation;
import com.jz.common.constant.CommConstant;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.dmp.modules.controller.dataService.bean.*;
import com.jz.dmp.modules.service.DmpApiMangeService;
import com.jz.dmp.modules.service.DmpOrgMangeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * 数据服务Api管理--调用gateway 接口
 *
 * @author Bellamy
 * @since 2021-01-18 10:56:18
 */
@RestController
@RequestMapping("/apiMange")
@Api(tags = "数据服务-Api管理")
public class DmpApiMangeController {

    private static Logger logger = LoggerFactory.getLogger(DmpApiMangeController.class);
    /**
     * 服务对象
     */
    @Autowired
    private DmpApiMangeService dmpApiMangeService;

    /*    *//**
     * 授权给他人的API-列表分页查询
     *
     * @return
     * @author Bellamy
     * @since 2021-01-18
     *//*
    @ApiOperation(value = "授权给他人的API-列表分页查询", notes = "列表分页查询")
    @PostMapping(value = "/apiAuthListPage")
    public JsonResult getApiAuthListPage(@RequestBody @Validated AuthListInfoReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.queryApiAuthListPage(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }*/

    /**
     * 取消授权
     *
     * @author Bellamy
     * @since 2021-01-18
     */
    @ApiOperation(value = "取消授权", notes = "取消授权")
    @GetMapping(value = "/cancelApiAuth")
    @ApiImplicitParam(name = "id", value = "授权id", required = true)
    @SysOperationLogAnnotation(actionType = CommConstant.OPT_TYPE_UNAUTH, descrption = "取消授权", optModular = CommConstant.MODULAR_DATASERVICE_AUTHMANAG, isProjOpt = true)
    public JsonResult getCancelApiAuth(@RequestParam String id, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(id)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "授权id不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.cancelApiAuth(id);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /* *//**
     * 发布的API-API列表分页查询
     *
     * @author Bellamy
     * @since 2021-01-18
     *//*
    @ApiOperation(value = "发布的API-API列表分页查询", notes = "列表分页查询")
    @PostMapping(value = "/listPage")
    public JsonResult getApiListPage(@RequestBody @Validated ApiInterfaceInfoListReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.queryApiListPage(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }*/

    /**
     * 删除api
     *
     * @author Bellamy
     * @since 2021-01-18
     */
    @ApiOperation(value = "删除api", notes = "删除api")
    @GetMapping(value = "/delApiInfo")
    @ApiImplicitParams({@ApiImplicitParam(name = "apiKey", value = "apiKey", required = true),
            @ApiImplicitParam(name = "type", value = "类型：0删除提示，1正式删除", required = true)})
    @SysOperationLogAnnotation(actionType = CommConstant.OPT_TYPE_DELETE, descrption = "删除api", optModular = CommConstant.MODULAR_DATASERVICE_DEV, isProjOpt = true)
    public JsonResult delApiInfo(@RequestParam String apiKey, @RequestParam String type, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(apiKey)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "apiKey不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.delApiInfo(apiKey, type);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 根据apiid获取API详情
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @ApiOperation(value = "获取API详情", notes = "获取API详情")
    @GetMapping(value = "/getApiInfo")
    @ApiImplicitParam(name = "apiKey", value = "apiKey", required = true)
    public JsonResult getApiInfoByApiId(@RequestParam String apiKey, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(apiKey)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "apiKey不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.queryApiInfoByApiId(apiKey);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /*  *//**
     * 授权给组织-提交保存
     *
     * @author Bellamy
     * @since 2021-01-19
     *//*
    @ApiOperation(value = "授权给组织-提交保存", notes = "授权给组织-提交保存")
    @PostMapping(value = "/addApiAuthToOrg")
    public JsonResult addApiAuthToOrg(@RequestBody @Validated AuthUserApiReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.addApiAuthToOrg(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }*/

    /**
     * API测试
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @ApiOperation(value = "API测试", notes = "API测试")
    @PostMapping(value = "/apiTestInfo")
    public JsonResult getApiTestInfo(@RequestBody Map<String, String> apiKey, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.apiTestInfo(apiKey);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * API测试--下拉框
     *
     * @author Bellamy
     * @since 2021-03-03
     */
    @ApiOperation(value = "API测试--下拉框", notes = "API测试--下拉框")
    @GetMapping(value = "/apiTestGetApiKey")
    @ApiImplicitParam(name = "apiName", value = "apiName")
    public JsonResult getApiTestGetApiKey(@RequestParam(required = false) String apiName, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.getApiTestGetApiKey(apiName);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * API日志列表
     *
     * @author Bellamy
     * @since 2021-01-19
     */
    @ApiOperation(value = "API日志列表", notes = "API日志列表")
    @PostMapping(value = "/checkApiLogInfo")
    public JsonResult checkApiLogInfo(@RequestBody LogInfoListReq req, HttpServletRequest httpRequest) {
       /* if (StringUtils.isEmpty(req.getApiKey())) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "apiKey不能为空！");
        }*/
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.checkApiLogInfo(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /*   *//**
     * 授权模糊查询组织信息
     *
     * @author Bellamy
     * @since 2021-01-19
     *//*
    @ApiOperation(value = "授权模糊查询组织信息", notes = "授权模糊查询组织信息")
    @GetMapping(value = "/authOrgList")
    @ApiImplicitParam(name = "key", value = "key", required = true)
    public JsonResult getAuthOrgList(@RequestParam String key, HttpServletRequest httpRequest) {
        if (StringUtils.isEmpty(key)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "key不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.getAuthOrgList(key);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }*/

    /**
     * 服务发布/服务取消
     *
     * @author Bellamy
     * @since 2021-02-07
     */
    @ApiOperation(value = "服务发布/服务取消", notes = "服务发布/服务取消")
    @GetMapping(value = "/apiPublishCancel")
    @ApiImplicitParam(name = "id", value = "id", required = true)
    @SysOperationLogAnnotation(actionType = CommConstant.OPT_TYPE_PUBLISHORUN, descrption = "服务发布/服务取消", optModular = CommConstant.MODULAR_DATASERVICE_DEV, isProjOpt = true)
    public JsonResult apiPublishCancel(@RequestParam String id, @RequestParam(name = "optStatus") Boolean optStatus) {
        if (StringUtils.isEmpty(id)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "id不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.apiPublishCancel(id, optStatus);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 盐值重置
     *
     * @author Bellamy
     * @since 2021-02-20
     */
    @ApiOperation(value = "盐值重置", notes = "盐值重置")
    @PostMapping(value = "/saltReset")
    @SysOperationLogAnnotation(actionType = CommConstant.OPT_TYPE_EDIT, descrption = "盐值重置", optModular = CommConstant.MODULAR_DATASERVICE_AUTHMANAG, isProjOpt = true)
    public JsonResult saltReset(@RequestBody @Validated SaltResetReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.saltReset(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 授权管理-api列表
     *
     * @author Bellamy
     * @since 2021-02-20
     */
    @ApiOperation(value = "授权管理-api列表", notes = "授权管理-api列表")
    @PostMapping(value = "/authApiList")
    public JsonResult authApiList(@RequestBody @Validated AuthApiListReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.queryAuthApiList(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 用户授权
     *
     * @author Bellamy
     * @since 2021-02-20
     */
    @ApiOperation(value = "授权", notes = "授权")
    @PostMapping(value = "/authUserApi")
    @SysOperationLogAnnotation(actionType = CommConstant.OPT_TYPE_AUTH, descrption = "授权", optModular = CommConstant.MODULAR_DATASERVICE_DEV, isProjOpt = true)
    public JsonResult authUserApi(@RequestBody @Validated AuthUserApiReq req, HttpServletRequest httpRequest) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.authUserApi(req);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 服务开发--上传（发送给数据银行）
     *
     * @author Bellamy
     * @since 2021-02-20
     */
    @ApiOperation(value = "服务开发--上传", notes = "上传")
    @GetMapping(value = "/serverUpload")
    @ApiImplicitParam(name = "id", value = "id", required = true)
    @SysOperationLogAnnotation(actionType = CommConstant.OPT_TYPE_UPLOAD, descrption = "服务开发--上传", optModular = CommConstant.MODULAR_DATASERVICE_DEV, isProjOpt = true)
    public JsonResult serverUpload(@RequestParam String id) {
        if (StringUtils.isEmpty(id)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "id不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.serverUpload(id);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 日志详情
     *
     * @author Bellamy
     * @since 2021-02-20
     */
    @ApiOperation(value = "日志详情", notes = "日志详情")
    @GetMapping(value = "/logDetails")
    @ApiImplicitParam(name = "id", value = "id", required = true)
    public JsonResult logDetails(@RequestParam String id) {
        if (StringUtils.isEmpty(id)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "id不能为空！");
        }
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.logDetails(id);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

    /**
     * 授权--组织名称查询
     *
     * @author Bellamy
     * @since 2021-03-02
     */
    @ApiOperation(value = "授权--组织名称查询", notes = "授权--组织名称查询")
    @GetMapping(value = "/getAuthOrgName")
    @ApiImplicitParams({@ApiImplicitParam(name = "orgName", value = "组织名称", required = true),
            @ApiImplicitParam(name = "projectId", value = "projectId", required = true)})
    public JsonResult getAuthOrgName(@RequestParam String orgName, String projectId) {
        JsonResult jsonResult = new JsonResult();
        try {
            jsonResult = dmpApiMangeService.getAuthOrgName(orgName, projectId);
        } catch (Exception e) {
            jsonResult.setMessage(e.getMessage());
            jsonResult.setCode(ResultCode.INTERNAL_SERVER_ERROR);
            e.printStackTrace();
        }
        return jsonResult;
    }

}