package com.jz.dmp.modules.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.amazonaws.services.dynamodbv2.xspec.M;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.agent.service.DmpDsAgentService;
import com.jz.common.constant.DatasouceTypeConstant;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.enums.DatasourceTypeEnum;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.CommonUtils;
import com.jz.common.utils.JsonMapper;
import com.jz.common.utils.realTime.CmdUtils;
import com.jz.common.utils.realTime.DBUtil;
import com.jz.common.utils.realTime.RestClient;
import com.jz.common.utils.web.HttpClientUtils;
import com.jz.common.utils.web.OKHttpUtil;
import com.jz.common.utils.web.SessionUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.modules.controller.DataIntegration.bean.*;
import com.jz.dmp.modules.dao.*;
import com.jz.dmp.modules.model.*;
import com.jz.dmp.modules.service.DmpRealtimeSyncInfoService;
import com.squareup.okhttp.OkHttpClient;
import freemarker.core.ParseException;
import freemarker.template.MalformedTemplateNameException;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateNotFoundException;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfig;

import java.io.IOException;
import java.io.StringWriter;
import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Pattern;

/**
 * 实时同步任务(DmpRealtimeSyncInfo)表服务实现类
 *
 * @author Bellamy
 * @since 2021-01-05 14:18:03
 */
@Service("dmpRealtimeSyncInfoService")
@Transactional
public class DmpRealtimeSyncInfoServiceImpl implements DmpRealtimeSyncInfoService {

    private static Logger logger = LoggerFactory.getLogger(DmpRealtimeSyncInfoServiceImpl.class);

    @Value("${spring.public-key}")
    private String publicKey;

    @Autowired
    private FreeMarkerConfig freeMarkerConfig;

    @Autowired
    private DmpRealtimeSyncInfoDao dmpRealtimeSyncInfoDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    @Autowired
    private DmpRealtimeSyncHandleCountDao dmpRealtimeSyncHandleCountDao;

    @Autowired
    private DmpNavigationTreeDao dmpNavigationTreeDao;

    @Autowired
    private OfflineSynchDao offlineSynchDao;

    @Autowired
    private DmpDsAgentService dmpDsAgentServiceImp;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @Override
    public DmpRealtimeSyncInfo queryById(Integer id) {
        return this.dmpRealtimeSyncInfoDao.queryById(id);
    }

    /**
     * 查询多条数据
     *
     * @param offset 查询起始位置
     * @param limit  查询条数
     * @return 对象列表
     */
    @Override
    public List<DmpRealtimeSyncInfo> queryAllByLimit(int offset, int limit) {
        return this.dmpRealtimeSyncInfoDao.queryAllByLimit(offset, limit);
    }

    /**
     * 新增数据
     *
     * @param dmpRealtimeSyncInfo 实例对象
     * @return 实例对象
     */
    @Override
    public DmpRealtimeSyncInfo insert(DmpRealtimeSyncInfo dmpRealtimeSyncInfo) {
        this.dmpRealtimeSyncInfoDao.insert(dmpRealtimeSyncInfo);
        return dmpRealtimeSyncInfo;
    }

    /**
     * 修改数据
     *
     * @param dmpRealtimeSyncInfo 实例对象
     * @return 实例对象
     */
    @Override
    public DmpRealtimeSyncInfo update(DmpRealtimeSyncInfo dmpRealtimeSyncInfo) {
        this.dmpRealtimeSyncInfoDao.update(dmpRealtimeSyncInfo);
        return this.queryById(dmpRealtimeSyncInfo.getId());
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public boolean deleteById(Integer id) {
        return this.dmpRealtimeSyncInfoDao.deleteById(id) > 0;
    }

    /**
     * 实时同步任务列表分页查询
     *
     * @return
     * @author Bellamy
     */
    @Override
    public PageInfoResponse<RealTimeSyncListDto> queryRealTimeSyncListPage(RealTimeSyncListReq req) throws Exception {
        PageInfoResponse<RealTimeSyncListDto> pageInfoResponse = new PageInfoResponse<>();

        if (StringUtils.isNotEmpty(req.getSourceDatabaseName())) { //来源数据源名称
            req.setSourceDatabaseName(req.getSourceDatabaseName().trim());
        }
        if (StringUtils.isNotEmpty(req.getTargetDatabaseName())) { //目标源数据源名称
            req.setTargetDatabaseName(req.getTargetDatabaseName().trim());
        }
        req.setTreeName(req.getTreeName().trim());//任务名称
        /*if (StringUtils.isNotEmpty(req.getTreeId())) {
            //判断是否为整数 是整数返回true,否则返回false
            Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");
            if (pattern.matcher(req.getTreeId().trim()).matches()) {
                req.setTreeId(req.getTreeId()); //id
            } else {
                req.setTreeName(req.getTreeId().trim());//节点名称
                req.setTreeId("");
            }
        }*/

        PageHelper.startPage(req.getPageNum(), req.getPageSize());
        List<RealTimeSyncListDto> list = dmpRealtimeSyncInfoDao.queryRealTimeSyncListPage(req);
        PageInfo<RealTimeSyncListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 根据多个实时任务id查询任务信息
     *
     * @return
     * @author Bellamy
     */
    @Override
    public List<DmpRealtimeSyncInfo> queryListById(String[] ids) throws Exception {
        return dmpRealtimeSyncInfoDao.queryListById(ids);
    }

    /**
     * 获取数据源和目标数据源下拉框
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @Override
    public JsonResult<List<DataSourceNameListDto>> queryDatasourceNameList(String projectId, String type) throws Exception {
        List<DataSourceNameListDto> list = dmpRealtimeSyncInfoDao.queryDatasourceNameList(projectId, type);
        return new JsonResult(list);
    }

    /**
     * 新增--选择源表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-06
     */
    @Override
    public JsonResult<RealTimeSyncModel> queryTableInfoByParams(RealTimeTableInfoReq req) throws Exception {
        Map<String, Map<String, String>> paramsMap = null;
        //根据来源数据源id获取数据信息
        RealTimeSyncDataSourceModel sourceDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(req.getSrcDatasourceId());
        sourceDbInfo.setPassword(new BaseService().decode(sourceDbInfo.getPassword(), publicKey));

        Map map = new HashMap();
        map.put("srcDatasourceId", req.getSrcDatasourceId());
        map.put("targetDatasourceId", req.getTargetDatasourceId());
        //数据源对应的表详细信息  判断表的记录在同步任务中是否存在记录,如果已经存在,说明同步过
        List<Map> list = dmpRealtimeSyncInfoDao.queryRealTimeInfoByDataSourceId(map);
        if (list.size() > 0 && list != null) {
            paramsMap = new HashMap<>();
            for (Map item : list) {
                String srcTableName = item.get("srcTableName").toString();
                paramsMap.put(srcTableName, item);
            }
        }
        Map<String, String> blackMapSetting = null;
        //查询来源数据源的黑名单表
        Map blackTableMap = dmpRealtimeSyncInfoDao.queryBlackTableByDataSourceId(req.getSrcDatasourceId());
        if (blackTableMap.size() > 0 && blackTableMap != null) {
            String blacklistTable = blackTableMap.get("blacklistTable").toString();
            if (StringUtils.isNotEmpty(blacklistTable)) {
                String[] blacklistTableArr = blacklistTable.split(",");
                if (blacklistTableArr.length > 0 && blacklistTableArr != null) {
                    blackMapSetting = new HashMap<>();
                    for (String str : blacklistTableArr) {
                        blackMapSetting.put(str, str);
                    }
                }
            }
        }
        //来源数据源下的表信息
        List<RealTimeSyncTableModel> dataSourceTables = DBUtil.getDataSourceTables(sourceDbInfo, req.getToQueryTableName(), true, paramsMap, req.getBlacklistTables(), req.getSelectTablesName(), blackMapSetting);

        RealTimeSyncModel realTimeSyncModel = new RealTimeSyncModel();
        realTimeSyncModel.setSrcdatasourceId(Integer.valueOf(req.getSrcDatasourceId()));
        realTimeSyncModel.setDataSourceName(sourceDbInfo.getDatasourceName());
        realTimeSyncModel.setDbName(sourceDbInfo.getDbName());
        realTimeSyncModel.setTables(dataSourceTables); //来源数据源下的表信息
        return new JsonResult(realTimeSyncModel);
    }

    /**
     * 新增--选择目标信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @Override
    public JsonResult selectTargetDatasourceInfo(String projectId) throws Exception {
        DmpProjectSystemInfo dmpProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(projectId));
        String kafkaConnectUrl = dmpProjectSystemInfo.getKafkaConnectorUrl(); //kafka 连接信息
        String[] arr = kafkaConnectUrl.split(",");
        return new JsonResult(arr);
    }

    /**
     * 设置成黑名单时 ,检查是否有同步任务存在运行
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @Override
    public JsonResult queryRealTimeInfoByDataSourceId(String srcDatasourceId, String sourceTableName) throws Exception {
        Map map = new HashMap();
        map.put("srcDatasourceId", srcDatasourceId); //来源表 数据源id
        map.put("sourceTableName", sourceTableName); // 来源表
        List<Map> list = dmpRealtimeSyncInfoDao.queryRealTimeInfoByDataSourceId(map);
        boolean flag = false;
        if (list.size() > 0 && list != null) {
            flag = true;
        }
        return new JsonResult(flag);
    }

    /**
     * 保存实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult addRealTimeTask(Map<String, Object> params) throws Exception {
        String srcDataSourceId = params.get("srcDataSourceId").toString(); // 来源id
        String targetDataSourceId = params.get("targetDataSourceId").toString(); // 目标源id
        Long projectId = Long.valueOf(params.get("projectId").toString()); //项目id
        String connectorUrl = (String) params.get("connectorUrl"); //connectorUrl连接信息

        //根据来源数据源id获取数据信息
        RealTimeSyncDataSourceModel sourceDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(srcDataSourceId);
        if (sourceDbInfo == null) {
            throw new RuntimeException("来源数据设置信息不存在！");
        }
        sourceDbInfo.setPassword(new BaseService().decode(sourceDbInfo.getPassword(), publicKey));

        //根据目标数据源id获取数据信息
        RealTimeSyncDataSourceModel targetDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(targetDataSourceId);
        if (targetDbInfo == null) {
            throw new RuntimeException("目标数据源设置信息不存在！");
        }
        targetDbInfo.setPassword(new BaseService().decode(targetDbInfo.getPassword(), publicKey));

        //根据projectId查询项目系统配置信息
        DmpProjectSystemInfo dmpProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(projectId);
        params.put("connectorSecurityFlag", dmpProjectSystemInfo.getKerberosIsenable()); //安全验证开关,是否启用KERBEROS

        String kafkaConnectUrl = dmpProjectSystemInfo.getKafkaConnectorUrl(); //kafka 连接信息
        String[] arr = kafkaConnectUrl.split(",");
        connectorUrl = arr[0];
        if (connectorUrl.contains("@")) {
            connectorUrl = connectorUrl.split("@")[1];
        }
        //提交源到源的connector
        Long realtimeId = submitDatasource2DatasourceToConnector(projectId, sourceDbInfo, targetDbInfo, dmpProjectSystemInfo, connectorUrl, params);
        if (realtimeId == null) {
            throw new RuntimeException("保存失败！");
        } else {
            submitSink2tableConnector(realtimeId, projectId, sourceDbInfo, targetDbInfo, dmpProjectSystemInfo, connectorUrl, params);
        }
        //处理已选择的表信息
        //submitNoSelectTable(realtimeId, projectId, sourceDbInfo, targetDbInfo, dmpProjectSystemInfo, connectorUrl, params);
        Map map = new HashMap();
        map.put("id", realtimeId);
        return JsonResult.ok(map);
    }

    private void submitSink2tableConnector(Long realtimeId, Long projectId, RealTimeSyncDataSourceModel sourceDbInfo, RealTimeSyncDataSourceModel targetDbInfo, DmpProjectSystemInfo dmpProjectSystemInfo, String connectorUrl, Map<String, Object> params) throws Exception {
        DmpNavigationTree tree = dmpNavigationTreeDao.queryById(Integer.valueOf(params.get("treeId").toString()));
        String sourceName = tree.getName();
        Integer srcDataSourceId = sourceDbInfo.getId();
        Integer targetDataSourceId = targetDbInfo.getId();
        String taskId = String.valueOf(params.get("taskId"));
        String kafkaSchemaRegisterUrl = dmpProjectSystemInfo.getKafkaSchemaRegisterUrl();
        String endpoint = targetDbInfo.getEndpoint();
        String jdbcUrl = targetDbInfo.getJdbcUrl();

        String dbType = "";
        Map<String, String> sinkModel = new HashMap<>();
        String topicName = CommonUtils.getRealTimeTopicName(taskId);
        //sinkModel.put("sourceName",topicName);
        sinkModel.put("shareKey", "OdqhcqdzCHGrQupYIsLKkAb9UOvm332r");
        if (DatasourceTypeEnum.Kudu.getValue() == targetDbInfo.getDatasourceTypeId()) {
            dbType = DatasourceTypeEnum.Kudu.getCode().toLowerCase();
            sinkModel.put("kuduMaster", endpoint);
            String impalaUrl = jdbcUrl;
            if (StringUtils.isNotEmpty(impalaUrl)) {
                impalaUrl = impalaUrl.replace("jdbc:impala://", "");
                if (impalaUrl.contains("/")) {
                    impalaUrl = impalaUrl.substring(0, impalaUrl.indexOf("/"));
                }
                sinkModel.put("impalaUrl", impalaUrl);
            }
            sinkModel.put("impalaFqdn", targetDbInfo.getAccessId());
            sinkModel.put("registryUrl", kafkaSchemaRegisterUrl);
        } else if (DatasourceTypeEnum.HDFS.getValue() == targetDbInfo.getDatasourceTypeId()) {
            dbType = DatasourceTypeEnum.HDFS.getCode().toLowerCase();
            sinkModel.put("jdbcUrl", jdbcUrl);
            sinkModel.put("databaseName", targetDbInfo.getDbName());
            sinkModel.put("hiveMetastoreUris", endpoint);
            sinkModel.put("hdfsNamenodePrincipal", targetDbInfo.getAccessId());
            sinkModel.put("keytabLocation", targetDbInfo.getAccessKey());
        }
        sinkModel.put("connectorSecurityFlag", (String) params.get("connectorSecurityFlag")); //安全验证开关,是否启用KERBEROS

        List<Map<String, Object>> tables = (List<Map<String, Object>>) params.get("tables");
        if (tables.size() > 0 && tables != null) {
            for (Map<String, Object> str : tables) {
                Map<String, String> paramJson = new HashMap<>();
                String sourceTableName = String.valueOf(str.get("sourceTableName"));
                String targetTableName = String.valueOf(str.get("targetTableName"));
                String targetDbNameAndTableName = targetDbInfo.getDbName() + "." + targetTableName;
                paramJson.put("targetDbNameAndTableName", targetDbNameAndTableName);
                String desensitizationField = "";
                String arithmetic = "";
                List<Map<String, String>> fieldInfo = (List<Map<String, String>>) str.get("fieldInfo"); //脱敏字段，算法
                for (Map<String, String> field : fieldInfo) {
                    desensitizationField = "," + field.get("desensitizationField");
                    arithmetic = field.get("arithmetic");
                }
                paramJson.put("desensitizationField", desensitizationField.substring(1));
                paramJson.put("arithmetic", arithmetic);
                paramJson.putAll(sinkModel);
                String topic = sourceDbInfo.getDatasourceName() + "_" + topicName + "." + sourceDbInfo.getDbName() + "." + targetTableName;
                paramJson.put("topic", topic);
                paramJson.put("name", targetDbInfo.getDatasourceName() + "@" + topic);
                String jsonStr = freemakerJson(dbType + "_sink", paramJson);
                logger.info("=======send sink to kafka {}" + jsonStr);

                DmpRealtimeSyncInfo saveBody = new DmpRealtimeSyncInfo();
                saveBody.setSrcDatasourceId(srcDataSourceId);
                saveBody.setSrcDatasourceName(sourceDbInfo.getDatasourceName());
                saveBody.setSrcDatabaseType(sourceDbInfo.getDataSourceTypeName());
                saveBody.setSrcDatabaseName(sourceDbInfo.getDbName());
                saveBody.setTargetDatasourceId(targetDataSourceId);
                saveBody.setTargetDatasourceName(targetDbInfo.getDatasourceName());
                saveBody.setTargetDatabaseType(targetDbInfo.getDataSourceTypeName());
                saveBody.setTargetDatabaseName(targetDbInfo.getDbName());
                saveBody.setType(1);
                saveBody.setConnectorJsonData(jsonStr);
                saveBody.setProjectId(String.valueOf(projectId));
                saveBody.setConnectorUrl(connectorUrl);
                saveBody.setSrcTopicName(topic);
                saveBody.setSourceTypeName(sourceDbInfo.getDataSourceTypeName());
                saveBody.setTargetTypeName("Kafka");
                saveBody.setCreateTime(new Date());
                saveBody.setCrePerson(SessionUtils.getCurrentUserId());
                saveBody.setVersion("1.0");
                saveBody.setScriptJson(JSONObject.toJSONString(params));
                saveBody.setTreeId(params.get("treeId").toString());
                saveBody.setStoreType(dbType);
                saveBody.setParentId(Integer.valueOf(realtimeId.toString()));
                if (StringUtils.isNotEmpty(String.valueOf(params.get("taskId")))) {
                    DmpRealtimeSyncInfo param = new DmpRealtimeSyncInfo();
                    param.setParentId(Integer.valueOf(realtimeId.toString()));
                    //编辑时，先删除任务，再发布任务
                    List<DmpRealtimeSyncInfo> sinkTaskList = dmpRealtimeSyncInfoDao.queryAll(param);
                    if (sinkTaskList.size() > 0 && sinkTaskList != null) {
                        Map maps = new HashMap();
                        maps.put("dataStatus", "0");
                        String ids = "";
                        for (DmpRealtimeSyncInfo item : sinkTaskList) {
                            ids = "," + item.getId();
                            HttpClientUtils.httpDelete(connectorUrl + "/" + item.getConnectorJobId() + "/");
                        }
                        maps.put("ids", ids.substring(1).split(","));
                        dmpRealtimeSyncInfoDao.deleteByrealTaskId(maps);
                    }
                    //新增sink
                    Map<String, String> respData = publishTask2Kafka(connectorUrl, jsonStr, "");
                    saveBody.setConnectorJobId(respData.get("connectorJobId"));
                    saveBody.setStatus(respData.get("status"));
                    dmpRealtimeSyncInfoDao.insert(saveBody);
                }
            }
        }
    }


    private Long submitDatasource2DatasourceToConnector(Long projectId, RealTimeSyncDataSourceModel sourceDbInfo, RealTimeSyncDataSourceModel targetDbInfo, DmpProjectSystemInfo dmpProjectSystemInfo, String connectorUrl, Map<String, Object> params) throws Exception {
        DmpNavigationTree tree = dmpNavigationTreeDao.queryById(Integer.valueOf(params.get("treeId").toString()));
        Integer srcDataSourceId = sourceDbInfo.getId();
        Integer targetDataSourceId = targetDbInfo.getId();

        Long realtiemId = null; //同步任务id
        //源数据源到数据源同步connector信息
        //解析黑名单表
        //Map blacklistTablesInfo = getBlackListTableInfo(sourceDbInfo, params);

        //解析已选择表
        Map selectlistTablesInfo = getSelectListTableInfo(sourceDbInfo, params);

        Map<String, String> dataModelMap = new HashMap<>();
        dataModelMap.put("kafkaBootstrapServers", dmpProjectSystemInfo.getKafkaBootstrapServers());//kafka 地址
        dataModelMap.put("registerUrl", dmpProjectSystemInfo.getKafkaSchemaRegisterUrl()); //kafka connector schema 注册地址
        //dataModelMap.put("blacklistTables", blacklistTablesInfo.get("connectorBlacklistTables").toString()); //设置的黑名单表
        //dataModelMap.put("blacklistTableCount", blacklistTablesInfo.get("blacklistTableCount").toString()); //黑名单表数量
        //设置的白名单表   在模板里进行判比较黑名单表和白名单表的数量，谁小就用谁
        dataModelMap.put("whitelistTablesConut", selectlistTablesInfo.get("whitelistTablesConut").toString()); //已选择表数量
        dataModelMap.put("connectorWhitelistTables", selectlistTablesInfo.get("connectorWhitelistTables").toString()); //设置的已选择表
        //选择的来源数据信息
        dataModelMap.put("dbHost", sourceDbInfo.getHost());
        dataModelMap.put("dbPort", sourceDbInfo.getPort());
        dataModelMap.put("dbUserName", sourceDbInfo.getUserName());
        dataModelMap.put("dbPassWord", sourceDbInfo.getPassword());
        dataModelMap.put("dataSourceId", sourceDbInfo.getId().toString());
        dataModelMap.put("datasourceName", sourceDbInfo.getDatasourceName());
        dataModelMap.put("dbName", sourceDbInfo.getDbName());
        dataModelMap.put("connectorSecurityFlag", (String) params.get("connectorSecurityFlag")); //安全验证开关,是否启用KERBEROS
        String sourceName = tree.getName();
        dataModelMap.put("sourceName", sourceName);
        //source kafak topic
        String topic = sourceDbInfo.getDatasourceName() + "_" + sourceName + "." + sourceDbInfo.getDbName() + ".databasehistory";
        dataModelMap.put("topic", topic);
        //connector_job_id 连接名称
        String sourceConnectorName = "debezium-connector-" + sourceDbInfo.getDatasourceName() + "_" + sourceName + "-" + sourceDbInfo.getDbName();
        //source connector name
        dataModelMap.put("name", sourceDbInfo.getDatasourceName() + "_" + sourceName + "-" + sourceDbInfo.getDbName());
        /*
         * 查询同步任务信息表是否存在类型是数据源到数据源，
         * srcDataSourceId，targetDataSourceId一样的信息如果 存在就不发起请求
         * */
        if (StringUtils.isEmpty(String.valueOf(params.get("taskId")))) {
            Map<String, Object> queryParam = new HashMap<>();
            queryParam.put("srcDataSourceId", srcDataSourceId);
            queryParam.put("sourceConnectorName", sourceConnectorName);
            Map dataMap = dmpRealtimeSyncInfoDao.selecltRealtimeSyncInfoByParams(queryParam);
            if (dataMap != null) {
                throw new RuntimeException("存在相同的实时任务数据！");
            }
        }
        //使用freemaker模板生成 kafka connector 请求参数
        String jsonStr = freemakerJson("source", dataModelMap);

        DmpRealtimeSyncInfo saveBody = new DmpRealtimeSyncInfo();
        saveBody.setSrcDatasourceId(srcDataSourceId);
        saveBody.setSrcDatasourceName(sourceDbInfo.getDatasourceName());
        saveBody.setSrcDatabaseType(sourceDbInfo.getDataSourceTypeName());
        saveBody.setSrcDatabaseName(sourceDbInfo.getDbName());
        saveBody.setTargetDatasourceId(targetDataSourceId);
        saveBody.setTargetDatasourceName(targetDbInfo.getDatasourceName());
        saveBody.setTargetDatabaseType(targetDbInfo.getDataSourceTypeName());
        saveBody.setTargetDatabaseName(targetDbInfo.getDbName());
        saveBody.setType(1);
        saveBody.setConnectorJsonData(jsonStr);
        saveBody.setProjectId(String.valueOf(projectId));
        saveBody.setConnectorUrl(connectorUrl);
        saveBody.setSrcTopicName(topic);
        saveBody.setSourceTypeName(sourceDbInfo.getDataSourceTypeName());
        saveBody.setTargetTypeName("Kafka");
        saveBody.setCreateTime(new Date());
        saveBody.setCrePerson(SessionUtils.getCurrentUserId());
        saveBody.setVersion("1.0");
        saveBody.setScriptJson(JSONObject.toJSONString(params));
        saveBody.setTreeId(params.get("treeId").toString());
        //版本记录
        DmpRealtimeTaskHistory taskHistory = new DmpRealtimeTaskHistory();
        BeanUtils.copyProperties(saveBody, taskHistory);
        if (StringUtils.isEmpty(String.valueOf(params.get("taskId")))) {
            Map<String, String> respData = publishTask2Kafka(connectorUrl, jsonStr, "");
            saveBody.setConnectorJobId(respData.get("connectorJobId"));
            saveBody.setStatus(respData.get("status"));
            dmpRealtimeSyncInfoDao.insert(saveBody);

            //异步获取任务状态
            Thread thread = new Thread(new Runnable() {
                @Override
                public void run() {
                    Map map = new HashMap();
                    map.put("id", saveBody.getId());
                    getKafkaTaskStatus(connectorUrl + respData.get("connectorJobId") + "/status", map);
                }
            });
            thread.start();
        } else {
            DmpRealtimeSyncInfo realtimeTask = dmpRealtimeSyncInfoDao.queryById(Integer.valueOf(params.get("taskId").toString()));
            if (realtimeTask == null)
                throw new RuntimeException("操作数据不存在！");

            //编辑时，先删除任务，再发布任务
            HttpClientUtils.httpDelete(connectorUrl + "/" + realtimeTask.getConnectorJobId() + "/");
            Map<String, String> respData = publishTask2Kafka(connectorUrl, jsonStr, params.get("taskId").toString());
            saveBody.setConnectorJobId(respData.get("connectorJobId"));
            saveBody.setStatus(respData.get("status"));
            saveBody.setUpdateTime(new Date());
            saveBody.setUptPerson(SessionUtils.getCurrentUserId());
            saveBody.setId(Integer.valueOf(params.get("taskId").toString()));
            dmpRealtimeSyncInfoDao.update(saveBody);

            DmpRealtimeSyncInfo devTask = dmpRealtimeSyncInfoDao.queryById(saveBody.getId());
            BigDecimal version = new BigDecimal(devTask.getVersion());
            version = version.add(new BigDecimal(1.0));
            taskHistory.setVersion(String.valueOf(version));
        }
        logger.info("################### save task end ################");

        taskHistory.setRealtimeSyncId(saveBody.getId());
        dmpRealtimeSyncInfoDao.insertRealtimeHistory(taskHistory);

        /*Map blacklist = new HashMap();
        blacklist.put("creTime", new Date());
        blacklist.put("realtimeId", saveBody.getId());
        blacklist.put("crePerson", SessionUtils.getCurrentUserId());
        blacklist.put("datasourceId", srcDataSourceId);
        blacklist.put("blacklistTable", blacklistTablesInfo.get("blacklistTables").toString());
        dmpRealtimeSyncInfoDao.insertRealtimeBlackList(blacklist);
        logger.info("###################保存实时同步黑名单数据--结束 ################");*/
        return Long.valueOf(saveBody.getId());
    }

    /*
     * 发布到kafak 并开始运行 ，请求接口正常则保存数据，否则失败
     * */
    public Map<String, String> publishTask2Kafka(String connectorUrl, String jsonStr, String id) throws Exception {
        Map<String, String> returnMap = new HashMap();
        Map<String, Object> result = RestClient.post(connectorUrl, jsonStr);
        logger.info("=======response data {}" + JSONObject.toJSONString(result));
        String connectorJobId = getConnectorJobId(result);
        if (StringUtils.isEmpty(connectorJobId)) {
            throw new RuntimeException("提交失败！");
        }
        Map params = new HashMap();
        params.put("id", id);
        params.put("flag", "publish");
        String status = getExecuteKafkaStatus(connectorUrl + "/" + connectorJobId + "/status", params);
        returnMap.put("status", status);
        returnMap.put("connectorJobId", connectorJobId);
        return returnMap;
    }

    /**
     * 解析黑名单表
     *
     * @return
     * @author Bellamy
     * @since 2021-01-11
     */
    public Map getBlackListTableInfo(RealTimeSyncDataSourceModel sourceDbInfo, Map<String, Object> params) {
        Map returnMap = new HashMap();
        String blacklistTables = "";
        String connectorBlacklistTables = "";
        int blacklistTableCount = 0;
        if (params.containsKey("blacklistTables")) {
            blacklistTables = (String) params.get("blacklistTables");
            if (StringUtils.isNotEmpty(blacklistTables)) {
                StringBuffer sb = new StringBuffer();
                String[] blacklistTableArr = blacklistTables.split(",");
                blacklistTableCount = blacklistTableArr.length;
                if (blacklistTableArr != null && blacklistTableCount >= 1) {
                    for (String string : blacklistTableArr) {
                        sb.append(sourceDbInfo.getDbName());
                        sb.append(".");
                        sb.append(string);
                        sb.append(",");
                    }
                }
                if (sb.toString().contains(",")) {
                    connectorBlacklistTables = sb.toString().substring(0, sb.toString().length() - 1);
                }
            }
        }
        returnMap.put("blacklistTables", blacklistTables);
        returnMap.put("connectorBlacklistTables", connectorBlacklistTables);
        returnMap.put("blacklistTableCount", blacklistTableCount);
        logger.info("###################解析黑名单表--结束{}" + connectorBlacklistTables + " ################");
        return returnMap;
    }

    /**
     * 解析已选择表
     *
     * @return
     * @author Bellamy
     * @since 2021-01-11
     */
    public Map getSelectListTableInfo(RealTimeSyncDataSourceModel sourceDbInfo, Map<String, Object> params) {
        Map returnMap = new HashMap();

        String connectorWhitelistTables = "";
        int whitelistTablesConut = 0;
        if (params.containsKey("tables")) {
            List<Map<String, String>> tables = (List<Map<String, String>>) params.get("tables");
            if (!CollectionUtils.isEmpty(tables)) {
                whitelistTablesConut = tables.size();
                StringBuffer sb = new StringBuffer();
                for (Map<String, String> temp : tables) {
                    sb.append(sourceDbInfo.getDbName());
                    sb.append(".");
                    sb.append(temp.get("sourceTableName"));
                    sb.append(",");
                }
                if (sb.toString().contains(",")) {
                    connectorWhitelistTables = sb.toString().substring(0, sb.toString().length() - 1);
                }
            }
        }
        returnMap.put("connectorWhitelistTables", connectorWhitelistTables);
        returnMap.put("whitelistTablesConut", whitelistTablesConut);
        logger.info("###################解析已选择表--结束{} " + connectorWhitelistTables + "################");
        return returnMap;
    }

    /**
     * 使用freemaker模板生成 kafka connector 请求参数
     *
     * @param type      模板类型
     * @param dataModel 模板里定义的变量数据对象
     * @return
     * @author Bellamy
     */
    public String freemakerJson(String type, Map<String, String> dataModel) {
        StringWriter stringWriter = new StringWriter();
        try {

            Template template = freeMarkerConfig.getConfiguration().getTemplate(type + "_connector.ftl");
            if (template != null) {
                try {
                    template.process(dataModel, stringWriter);
                } catch (TemplateException e) {
                    e.printStackTrace();
                }
            }
        } catch (TemplateNotFoundException e) {
            e.printStackTrace();
        } catch (MalformedTemplateNameException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return stringWriter.toString();
    }

    /**
     * 根据返回结果  获取返回的connectorName
     *
     * @param resultMap
     * @return
     * @author Bellamy
     */
    public String getConnectorJobId(Map<String, Object> resultMap) {
        if (CollectionUtils.isEmpty(resultMap)) {
            return null;
        }
        if (!resultMap.containsKey("name")) {
            return null;
        }
        return (String) resultMap.get("name");
    }

    /**
     * 处理已选择的表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    private JsonResult submitNoSelectTable(Long realtimeId, Long projectId, RealTimeSyncDataSourceModel sourceDbInfo, RealTimeSyncDataSourceModel targetDbInfo, DmpProjectSystemInfo dmpProjectSystemInfo, String connectorUrl, Map<String, Object> params) throws Exception {
        logger.info("###################处理已选择的表信息################");

       /* //源数据源到数据源同步connector信息  ----开始------
        Map<String, String> tableToTableConfigArg = new HashMap<>();
        //默认配置参数  --{kudu master 的地址} 10.0.108.61,10.0.108.62
        String endpoint = targetDbInfo.getEndpoint();
        //--{share key} CRNET_SECRET_KEY
        String shareKey = (String) getDictionaryParamValueByKey("CRNET_SECRET_KEY");
        tableToTableConfigArg.put("shareKey", shareKey);
        //--{impala 地址} 10.0.108.62:21050
        //目标数据源类型
        String storeType = targetDbInfo.getDataSourceTypeName().toLowerCase();
        String jdbcUrl = targetDbInfo.getJdbcUrl();
        if ("kudu".equals(storeType)) {
            tableToTableConfigArg.put("kuduMaster", endpoint);
            String impalaUrl = jdbcUrl;
            if (!org.springframework.util.StringUtils.isEmpty(impalaUrl)) {
                impalaUrl = impalaUrl.replace("jdbc:impala://", "");
                if (impalaUrl.contains("/")) {
                    impalaUrl = impalaUrl.substring(0, impalaUrl.indexOf("/"));
                }
                tableToTableConfigArg.put("impalaUrl", impalaUrl);
            }
            tableToTableConfigArg.put("impalaFqdn", targetDbInfo.getAccessId());
            //--{schem registry url} http://10.0.108.61:9881
            tableToTableConfigArg.put("registryUrl", dmpProjectSystemInfo.getKafkaSchemaRegisterUrl());
        } else if ("hdfs".equals(storeType)) {
            tableToTableConfigArg.put("jdbcUrl", jdbcUrl);
            tableToTableConfigArg.put("databaseName", targetDbInfo.getDbName());
            tableToTableConfigArg.put("hiveMetastoreUris", endpoint);
            tableToTableConfigArg.put("hdfsNamenodePrincipal", targetDbInfo.getAccessId());
            tableToTableConfigArg.put("keytabLocation", targetDbInfo.getAccessKey());
        }
        tableToTableConfigArg.put("connectorSecurityFlag", (String) params.get("connectorSecurityFlag"));*/

        //已选择的表信息
        List<Map<String, String>> tables = (List<Map<String, String>>) params.get("tables");
        if (tables.size() > 0 && tables != null) {
            for (Map<String, String> str : tables) {
                DmpRealtimeSyncSelectTable selectTable = new DmpRealtimeSyncSelectTable();
                selectTable.setCrePerson(SessionUtils.getCurrentUserId());
                selectTable.setCreTime(new Date());
                selectTable.setSrcTableName(str.get("sourceTableName"));
                selectTable.setTargetTableName(str.get("targetTableName"));
                selectTable.setRealtimeId(realtimeId);
                selectTable.setDesensitizationField(str.get("desensitizationField"));
                selectTable.setPkName(str.get("pkName"));
                selectTable.setArithmetic(str.get("arithmetic"));
                dmpRealtimeSyncInfoDao.insertRealtimeSelectTable(selectTable);
            }
        }
        logger.info("###################处理已选择的表信息--结束################");
        return JsonResult.ok();
    }

    /**
     * 编辑实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-08
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult updateRealTimeTask(Map<String, Object> params) throws Exception {
        String srcDataSourceId = params.get("srcDataSourceId").toString(); //来源id
        String targetDataSourceId = params.get("targetDataSourceId").toString(); //目标源id
        Long projectId = Long.valueOf(params.get("projectId").toString()); //项目id
        String connectorUrl = (String) params.get("connectorUrl"); //connectorUrl连接信息

        //根据来源数据源id获取数据信息
        RealTimeSyncDataSourceModel sourceDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(srcDataSourceId);
        if (sourceDbInfo == null) {
            throw new RuntimeException("来源数据设置信息不存在！");
        }
        sourceDbInfo.setPassword(new BaseService().decode(sourceDbInfo.getPassword(), publicKey));

        //根据目标数据源id获取数据信息
        RealTimeSyncDataSourceModel targetDbInfo = dmpRealtimeSyncInfoDao.querygSourceDbInfoById(targetDataSourceId);
        if (targetDbInfo == null) {
            throw new RuntimeException("目标数据源设置信息不存在！");
        }
        targetDbInfo.setPassword(new BaseService().decode(targetDbInfo.getPassword(), publicKey));

        //根据projectId查询项目系统配置信息
        DmpProjectSystemInfo dmpProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(projectId);
        params.put("connectorSecurityFlag", dmpProjectSystemInfo.getKerberosIsenable()); //安全验证开关,是否启用KERBEROS
        String kafkaConnectUrl = dmpProjectSystemInfo.getKafkaConnectorUrl(); //kafka 连接信息
        String[] arr = kafkaConnectUrl.split(",");
        connectorUrl = arr[0];
        if (connectorUrl.contains("@")) {
            connectorUrl = connectorUrl.split("@")[1];
        }
        //提交源到源的connector
        JsonResult realtimeId = updateDatasource2DatasourceToConnector(projectId, sourceDbInfo, targetDbInfo, dmpProjectSystemInfo, connectorUrl, params);

        //编辑 已选择表信息
        //this.updateNoSelectTable(params);
        return JsonResult.ok();
    }

    private JsonResult updateDatasource2DatasourceToConnector(Long projectId, RealTimeSyncDataSourceModel sourceDbInfo, RealTimeSyncDataSourceModel targetDbInfo, DmpProjectSystemInfo dmpProjectSystemInfo, String connectorUrl, Map<String, Object> params) throws Exception {
        DmpNavigationTree tree = dmpNavigationTreeDao.queryById(Integer.valueOf(params.get("treeId").toString()));
        Integer srcDataSourceId = sourceDbInfo.getId(); //来源数据源id
        Integer targetDataSourceId = targetDbInfo.getId(); //目标数据源id

        String realtiemId = params.get("taskId").toString(); //同步任务id
        //解析黑名单表
        //Map blacklistTablesInfo = getBlackListTableInfo(sourceDbInfo, params);

        //解析已选择表
        Map selectlistTablesInfo = getSelectListTableInfo(sourceDbInfo, params);

        Map<String, String> dataModelMap = new HashMap<>();
        dataModelMap.put("kafkaBootstrapServers", dmpProjectSystemInfo.getKafkaBootstrapServers());//kafka 地址
        dataModelMap.put("registerUrl", dmpProjectSystemInfo.getKafkaSchemaRegisterUrl()); //kafka connector schema 注册地址
        //dataModelMap.put("blacklistTables", blacklistTablesInfo.get("connectorBlacklistTables").toString()); //设置的黑名单表
        //dataModelMap.put("blacklistTableCount", blacklistTablesInfo.get("blacklistTableCount").toString()); //黑名单表数量
        //设置的白名单表   在模板里进行判比较黑名单表和白名单表的数量，谁小就用谁
        dataModelMap.put("whitelistTablesConut", selectlistTablesInfo.get("whitelistTablesConut").toString()); //已选择表数量
        dataModelMap.put("connectorWhitelistTables", selectlistTablesInfo.get("connectorWhitelistTables").toString()); //设置的已选择表
        //选择的来源数据信息
        dataModelMap.put("dbHost", sourceDbInfo.getHost()); //IP
        dataModelMap.put("dbPort", sourceDbInfo.getPort()); //端口
        dataModelMap.put("dbUserName", sourceDbInfo.getUserName());
        dataModelMap.put("dbPassWord", sourceDbInfo.getPassword());
        dataModelMap.put("dataSourceId", sourceDbInfo.getId().toString());
        dataModelMap.put("datasourceName", sourceDbInfo.getDatasourceName()); //数据源名称
        dataModelMap.put("dbName", sourceDbInfo.getDbName()); //数据库名称
        dataModelMap.put("connectorSecurityFlag", (String) params.get("connectorSecurityFlag")); //安全验证开关,是否启用KERBEROS
        String sourceName = tree.getName();
        dataModelMap.put("sourceName", sourceName);
        //source kafak topic
        String topic = sourceDbInfo.getDatasourceName() + "_" + sourceName + "." + sourceDbInfo.getDbName() + ".databasehistory";
        dataModelMap.put("topic", topic);
        dataModelMap.put("name", sourceDbInfo.getDatasourceName() + "_" + sourceName + "-" + sourceDbInfo.getDbName()); //source connector name
        String dataSource2DataSourceJsonStr = freemakerJson("source", dataModelMap); //使用freemaker模板生成 kafka connector 请求参数

        Map<String, Object> dataSource2DataSourceResult = RestClient.post(connectorUrl, dataSource2DataSourceJsonStr);
        String connectorJobId = getConnectorJobId(dataSource2DataSourceResult);
        //请求接口正常则保存数据，否则失败
        if (StringUtils.isNotEmpty(connectorJobId)) {
            DmpRealtimeSyncInfo saveBody = new DmpRealtimeSyncInfo();
            saveBody.setSrcDatasourceId(srcDataSourceId);
            saveBody.setSrcDatasourceName(sourceDbInfo.getDatasourceName());
            saveBody.setSrcDatabaseType(sourceDbInfo.getDataSourceTypeName());
            saveBody.setSrcDatabaseName(sourceDbInfo.getDbName());
            saveBody.setTargetDatasourceId(targetDataSourceId);
            saveBody.setTargetDatasourceName(targetDbInfo.getDatasourceName());
            saveBody.setTargetDatabaseType(targetDbInfo.getDataSourceTypeName());
            saveBody.setTargetDatabaseName(targetDbInfo.getDbName());
            saveBody.setStatus("SUBMIT_SUCCESS");
            saveBody.setConnectorJsonData(dataSource2DataSourceJsonStr);
            saveBody.setProjectId(String.valueOf(projectId));
            saveBody.setConnectorUrl(connectorUrl);
            saveBody.setSrcTopicName(topic);
            saveBody.setSourceTypeName(sourceDbInfo.getDataSourceTypeName());
            saveBody.setTargetTypeName("Kafka");
            saveBody.setConnectorJobId(connectorJobId);
            saveBody.setConnectorJsonData(dataSource2DataSourceJsonStr);
            saveBody.setUpdateTime(new Date());
            saveBody.setId(Integer.valueOf(realtiemId));
            saveBody.setUptPerson(SessionUtils.getCurrentUserId());
            //saveBody.setScriptJson();
            dmpRealtimeSyncInfoDao.update(saveBody);
            logger.info("###################修改实时同步任务--结束 ################");

            DmpRealtimeTaskHistory taskHistory = new DmpRealtimeTaskHistory();
            BeanUtils.copyProperties(saveBody, taskHistory);
            taskHistory.setRealtimeSyncId(saveBody.getId());
            taskHistory.setCrePerson(SessionUtils.getCurrentUserId());
            taskHistory.setCreateTime(new Date());
            dmpRealtimeSyncInfoDao.insertRealtimeHistory(taskHistory);

           /* Map blacklist = new HashMap();
            blacklist.put("uptTime", new Date());
            blacklist.put("uptPerson", SessionUtils.getCurrentUserId());
            blacklist.put("realtimeId", realtiemId);
            blacklist.put("datasourceId", srcDataSourceId);
            blacklist.put("blacklistTable", blacklistTablesInfo.get("blacklistTables").toString());
            dmpRealtimeSyncInfoDao.updateRealtimeBlackList(blacklist);
            logger.info("###################修改实时同步黑名单数据--结束 ################");*/
        } else {
            throw new RuntimeException("提交失败！");
        }
        return new JsonResult();
    }

    /**
     * 修改处理已选择的表信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-11
     */
    private JsonResult updateNoSelectTable(Map<String, Object> params) throws Exception {
        logger.info("###################处理已选择的表信息################");
        String realtimeId = params.get("taskId").toString(); //同步任务id
        //已选择的表信息
        List<Map<String, String>> tables = (List<Map<String, String>>) params.get("tables");
        if (tables.size() > 0 && tables != null) {
            for (Map<String, String> str : tables) {
                DmpRealtimeSyncSelectTable selectTable = new DmpRealtimeSyncSelectTable();
                selectTable.setUptPerson(SessionUtils.getCurrentUserId());
                selectTable.setUptTime(new Date());
                selectTable.setSrcTableName(str.get("sourceTableName"));
                selectTable.setTargetTableName(str.get("targetTableName"));
                selectTable.setRealtimeId(Long.valueOf(realtimeId));
                selectTable.setDesensitizationField(str.get("desensitizationField"));
                selectTable.setPkName(str.get("pkName"));
                selectTable.setArithmetic(str.get("arithmetic"));
                dmpRealtimeSyncInfoDao.updateRealtimeSelectTable(selectTable);
            }
        }
        logger.info("###################处理已选择的表信息--结束################");
        return new JsonResult();
    }

    /**
     * 编辑--数据回显,通过id查询任务相关信息
     *
     * @return
     * @author Bellamy
     * @since 2021-01-12
     */
    @Override
    public JsonResult<RealTimeEditDataEchoDto> selectRealtimeTaskById(String taskId) throws Exception {
        RealTimeEditDataEchoDto returnModel = new RealTimeEditDataEchoDto();
        List<Map> list = dmpRealtimeSyncInfoDao.selectRealtimeTaskById(taskId);
        if (list.size() > 0 && list != null) {
            Map realtimeMap = list.get(0);
            returnModel.setSrcDataSourceId(String.valueOf(realtimeMap.get("srcDatasourceId")));
            returnModel.setTargetDataSourceId(String.valueOf(realtimeMap.get("targetDatasourceId")));
            returnModel.setTreeId((String) realtimeMap.get("treeId"));
            returnModel.setId(realtimeMap.get("id").toString());
            Map map = (Map) JSONObject.parse((String) realtimeMap.get("scriptJson"));
            if (map != null && map.size() > 0) {
                returnModel.setRegularExpression((String) map.get("regularExpression"));
                if (map.containsKey("srcDatasourceTypeId")) {
                    returnModel.setSrcDatasourceTypeId(String.valueOf(map.get("srcDatasourceTypeId")));
                }
                if (map.containsKey("targetDataSourceTypeId")) {
                    returnModel.setTargetDataSourceTypeId(String.valueOf(map.get("targetDataSourceTypeId")));
                }
                if (map.containsKey("projectId")) {
                    returnModel.setProjectId(String.valueOf(map.get("projectId")));
                }
                returnModel.setTables((List<Map>) map.get("tables"));
            }
        }
        return new JsonResult(returnModel);
    }

    /**
     * 删除实时同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult deleteByrealTaskId(String realTaskId) throws Exception {
        DmpRealtimeSyncInfo realtimeTask = dmpRealtimeSyncInfoDao.queryById(Integer.valueOf(realTaskId));
        if (realtimeTask == null) {
            return JsonResult.error(ResultCode.OPERATION_DATA_NO_EXIST);
        }
        if (StringUtils.isEmpty(realtimeTask.getConnectorJobId())) {
            return JsonResult.error(ResultCode.OPERATION_DATA_NO_EXIST);
        }

        Map map = new HashMap();
        String[] ids = realTaskId.split(",");
        map.put("ids", ids);
        map.put("dataStatus", DelFlagEnum.YES.getValue());
        int len = dmpRealtimeSyncInfoDao.deleteByrealTaskId(map);
        if (len > 0) {
            map.put("ids", realtimeTask.getTreeId());
            dmpNavigationTreeDao.deleteByTreeId(map);
        }
        DmpProjectSystemInfo dmpProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(realtimeTask.getProjectId()));
        String kafkaConnectUrl = dmpProjectSystemInfo.getKafkaConnectorUrl(); //kafka 连接信息
        String[] arr = kafkaConnectUrl.split(",");
        String connectorUrl = arr[0];
        if (connectorUrl.contains("@")) {
            connectorUrl = connectorUrl.split("@")[1];
        }
        HttpClientUtils.httpDelete(connectorUrl + "/" + realtimeTask.getConnectorJobId() + "/");
        return JsonResult.ok();
    }

    /**
     * 批量上下线
     *
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @Override
    public boolean batchUptOnlineStatus(String realTaskId, String onlineStatus) throws Exception {
        Map map = new HashMap();
        String[] ids = realTaskId.split(",");
        map.put("ids", ids);
        map.put("onlineStatus", onlineStatus);
        return dmpRealtimeSyncInfoDao.deleteByrealTaskId(map) > 0;
    }

    /**
     * 冲突检查(选择源表在目标表中存在就冲突)
     *
     * @param params
     * @return
     * @author Bellamy
     * @since 2021-02-22
     */
    @Override
    public JsonResult conflictCheck(ConflictCheckReq params) throws Exception {
        List<Map> returnList = new ArrayList<>();
        //通过源数据库id ，查询数据源配置
        DmpAgentDatasourceInfo dsInfo = offlineSynchDao.querySourceDbInfoBySourceId(Long.valueOf(params.getTargetDataSourceId()));
        if (dsInfo == null) {
            throw new RuntimeException("数据源配置信息不存在！");
        }
        //解码源数据库密码
        if (StringUtils.isNotBlank(dsInfo.getPassword())) {
            dsInfo.setPassword(new BaseService().decode(dsInfo.getPassword(), publicKey));
        }
        //创建jdbc,获取源数据表
        DmpAgentResult rst = dmpDsAgentServiceImp.getTableNameList(dsInfo);
        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        } else {
            //成功
            List<String> tableList = (List<String>) JSONObject.parse(rst.getMessage());

            String[] tablesName = params.getTablesName().split(",");
            if (tablesName.length > 0) {
                for (String table : tablesName) {
                    Map map = new HashMap();
                    map.put("tableName", table);
                    map.put("sourceTableName", table);
                    map.put("targetTableName", table);
                    if (tableList.contains(table)) {
                        map.put("conflict", "Y");
                    } else {
                        map.put("conflict", "N");
                    }
                    returnList.add(map);
                }
            }
        }
        return JsonResult.ok(returnList);
    }

    /**
     * 启动/停止运行实时同步任务
     *
     * @param realTaskId
     * @param type
     * @return
     * @author Bellamy
     * @since 2021-01-05
     */
    @Override
    public JsonResult executeRealtimeTask(String realTaskId, String type, String projectId) throws Exception {
        DmpProjectSystemInfo dmpProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(Long.valueOf(projectId));
        String kafkaConnectUrl = dmpProjectSystemInfo.getKafkaConnectorUrl(); //kafka 连接信息
        String[] arr = kafkaConnectUrl.split(",");
        String connectorUrl = arr[0];
        if (connectorUrl.contains("@")) {
            connectorUrl = connectorUrl.split("@")[1];
        }
        String[] ids = realTaskId.split(",");
        List<DmpRealtimeSyncInfo> list = queryListById(ids);
        if (list.size() > 0 && list != null) {
            for (int i = 0; i < list.size(); i++) {
                DmpRealtimeSyncInfo saveBaby = new DmpRealtimeSyncInfo();
                DmpRealtimeSyncInfo dmpRealtimeSyncInfo = list.get(i);
                String connectorJobId = dmpRealtimeSyncInfo.getConnectorJobId();
                System.out.println(connectorJobId);
                String url = connectorUrl + "/" + connectorJobId;
                String statusUrl = url;
                //resume 恢复，删除 delete,pause 暂停
                if ("01".equals(type)) {
                    url += "/resume";
                } else if ("02".equals(type)) {
                    url += "/pause";
                }
                logger.info("######执行表数据id{}" + ids[i]);

                //请求 kafka api
                OKHttpUtil.httpPut(url, "");
                //获取任务状态
                Map params = new HashMap<>();
                params.put("id", realTaskId);
                params.put("flag", url.substring(url.lastIndexOf("/") + 1));
                String status = getExecuteKafkaStatus(statusUrl + "/status", params);

                //执行后任务状态 :PAUSED 暂停 , RUNNING 运行中
                saveBaby.setStatus(status);
                saveBaby.setUptPerson(SessionUtils.getCurrentUserId());
                saveBaby.setUpdateTime(new Date());
                saveBaby.setId(Integer.valueOf(realTaskId));
                dmpRealtimeSyncInfoDao.update(saveBaby);
            }
        }
        return JsonResult.ok();
    }

    /**
     * 通过treeId 查询实时任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-12
     */
    @Override
    public JsonResult queryRealtimeTaskInfoByTreeId(String treeId) throws Exception {
        Map map = dmpRealtimeSyncInfoDao.queryRealtimeTaskInfoByTreeId(treeId);
        return JsonResult.ok(map);
    }

    /*
     * kafka rest api 获取任务状态
     * */
    public String getExecuteKafkaStatus(String statusUrl, Map params) throws Exception {
        String status = "";
        String respData = HttpClientUtils.getJsonForParam(statusUrl, new HashMap<>());
        if (StringUtils.isEmpty(respData)) {
            throw new RuntimeException("get status failed！");
        }

        Map map = JSONObject.parseObject(respData);
        Map connector = (Map) map.get("connector");
        if (null != connector && connector.size() > 0) {
            if (!connector.containsKey("state")) {
                throw new RuntimeException("get status failed！");
            }
            status = (String) connector.get("state");
            logger.info("response connector status{}" + status);
        }

        if (StringUtils.isNotEmpty((String) params.get("id"))) {
            Thread thread = new Thread(new Runnable() {
                @Override
                public void run() {
                    getKafkaTaskStatus(statusUrl, params);
                }
            });
            thread.start();
        }
        return status;
    }

    /*
     * kafka rest api 获取任务状态
     * */
    public String getKafkaTaskStatus(String statusUrl, Map params) {
        String respData = HttpClientUtils.getJsonForParam(statusUrl, null);
        if (StringUtils.isEmpty(respData)) {
            throw new RuntimeException("get status failed！");
        }
        Map map = JSONObject.parseObject(respData);
        List<Map> tasks = (List<Map>) map.get("tasks");
        if (tasks.size() == 0) {
            String s = getKafkaTaskStatus(statusUrl, params);
            if (StringUtils.isNotEmpty(s)) {
                return s;
            }
        }
        String status = (String) tasks.get(0).get("state");
        logger.info("response tasks status{}" + status);
        DmpRealtimeSyncInfo saveBody = new DmpRealtimeSyncInfo();
        saveBody.setStatus(status);
        saveBody.setId(Integer.valueOf(params.get("id").toString()));
        dmpRealtimeSyncInfoDao.update(saveBody);
        return status;
    }
}