package com.jz.dmp.modules.service.projconfig.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.constant.StatuConstant;
import com.jz.common.persistence.BaseService;
import com.jz.dmp.modules.controller.projconfig.bean.DmpComputEngineBatch;
import com.jz.dmp.modules.controller.projconfig.bean.DmpComputEngineDto;
import com.jz.dmp.modules.controller.projconfig.bean.DmpComputEngineRequest;
import com.jz.dmp.modules.dao.projconfig.DmpComputEngineMapper;
import com.jz.dmp.modules.model.DmpComputEngine;
import com.jz.dmp.modules.service.projconfig.DmpComputEngineService;

/**
 * 计算引擎表服务的实现?
 * 
 * @author ybz
 *
 */
@Service("dmpComputEngineService")
public class DmpComputEngineServiceImpl extends BaseService implements DmpComputEngineService {

	private static Logger logger = LoggerFactory.getLogger(DmpComputEngineServiceImpl.class);

	@Autowired
	private DmpComputEngineMapper dmpComputEngineMapper;

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#findList(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public PageInfoResponse<DmpComputEngineDto> findListWithPage(DmpComputEngineRequest dmpComputEngineRequest, HttpServletRequest httpRequest)
			throws Exception {
		PageInfoResponse<DmpComputEngineDto> pageInfoResponse = new PageInfoResponse<>();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpComputEngineRequest.getEngineId() != null) {
			param.put("engineId", dmpComputEngineRequest.getEngineId());
		}
		//引擎名称
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getEngineName())) {
			param.put("engineName", dmpComputEngineRequest.getEngineName());
		}
		//备注
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getRemark())) {
			param.put("remark", dmpComputEngineRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getDataStatus())) {
			param.put("dataStatus", dmpComputEngineRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpComputEngineRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpComputEngineRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpComputEngineRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpComputEngineRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpComputEngineRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpComputEngineRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpComputEngineRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpComputEngineRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpComputEngineRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpComputEngineRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpComputEngineRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpComputEngineRequest.getUpdateTimeEnd());
		}

		PageHelper.startPage(dmpComputEngineRequest.getPageNum(), dmpComputEngineRequest.getPageSize());
		List<DmpComputEngineDto> list = dmpComputEngineMapper.findList(param);
		PageInfo<DmpComputEngineDto> pageInfo = new PageInfo<>(list);

		pageInfoResponse.setCode(StatuConstant.SUCCESS_CODE);
		pageInfoResponse.setMessage("查询成功");
		pageInfoResponse.setData(pageInfo);
		return pageInfoResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#findList(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpComputEngineDto> findList(DmpComputEngineRequest dmpComputEngineRequest, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpComputEngineDto> baseBeanResponse = new BaseBeanResponse<>();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpComputEngineRequest.getEngineId() != null) {
			param.put("engineId", dmpComputEngineRequest.getEngineId());
		}
		//引擎名称
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getEngineName())) {
			param.put("engineName", dmpComputEngineRequest.getEngineName());
		}
		//备注
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getRemark())) {
			param.put("remark", dmpComputEngineRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getDataStatus())) {
			param.put("dataStatus", dmpComputEngineRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpComputEngineRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpComputEngineRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpComputEngineRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpComputEngineRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpComputEngineRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpComputEngineRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpComputEngineRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpComputEngineRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpComputEngineRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpComputEngineRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpComputEngineRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpComputEngineRequest.getUpdateTimeEnd());
		}

		List<DmpComputEngineDto> list = dmpComputEngineMapper.findList(param);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setDatas(list);
		return baseBeanResponse;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#add(com.ycxc.vmts.entity.DmpComputEngine,
	 * javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpComputEngine> add(DmpComputEngine dmpComputEngine, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpComputEngine> baseBeanResponse = new BaseBeanResponse<>();

		/*$*setEngineIdCodeReplace*$*/
		// 创建人
		dmpComputEngine.setCreateUserId(getHttpRequestUserId(httpRequest));
		// 创建时间
		dmpComputEngine.setCreateTime(new Date());

		dmpComputEngineMapper.insertSelective(dmpComputEngine);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("新增成功");
		baseBeanResponse.setData(dmpComputEngine);

		return baseBeanResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#edit(com.ycxc.vmts.entity.DmpComputEngine,
	 * javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpComputEngine> edit(DmpComputEngine dmpComputEngine, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpComputEngine> baseBeanResponse = new BaseBeanResponse<>();

		// 修改人
		dmpComputEngine.setUpdateUserId(getHttpRequestUserId(httpRequest));
		// 修改时间
		dmpComputEngine.setUpdateTime(new Date());

		dmpComputEngineMapper.updateByPrimaryKeySelective(dmpComputEngine);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("修改成功");
		baseBeanResponse.setData(dmpComputEngine);

		return baseBeanResponse;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#addOrEdit(com.ycxc.vmts.entity.DmpComputEngine,
	 * javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpComputEngine> addOrEdit(DmpComputEngine dmpComputEngine, HttpServletRequest httpRequest) throws Exception {
		BaseBeanResponse<DmpComputEngine> baseBeanResponse = new BaseBeanResponse<>();
		
		Integer engineId = dmpComputEngine.getEngineId();
		
		if (null==engineId) {//新增
			// 创建人
			dmpComputEngine.setCreateUserId(getHttpRequestUserId(httpRequest));
			// 创建时间
			dmpComputEngine.setCreateTime(new Date());

			dmpComputEngineMapper.insertSelective(dmpComputEngine);

			baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
			baseBeanResponse.setMessage("新增成功");
			baseBeanResponse.setData(dmpComputEngine);
		} else {//修改
			// 修改人
			dmpComputEngine.setUpdateUserId(getHttpRequestUserId(httpRequest));
			// 修改时间
			dmpComputEngine.setUpdateTime(new Date());

			dmpComputEngineMapper.updateByPrimaryKeySelective(dmpComputEngine);

			baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
			baseBeanResponse.setMessage("修改成功");
			baseBeanResponse.setData(dmpComputEngine);
		}
		
		return baseBeanResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#findById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	public BaseBeanResponse<DmpComputEngineDto> findById(Integer engineId, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpComputEngineDto> baseBeanResponse = new BaseBeanResponse<>();

		DmpComputEngineDto dmpComputEngineDto = dmpComputEngineMapper.findById(engineId);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("查询成功");
		baseBeanResponse.setData(dmpComputEngineDto);

		return baseBeanResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse deleteById(Integer engineId, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpComputEngineMapper.deleteByPrimaryKey(engineId);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse softDeleteById(Integer engineId, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpComputEngineMapper.softDeleteByPrimaryKey(engineId);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse delete(DmpComputEngineRequest dmpComputEngineRequest, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpComputEngineRequest.getEngineId() != null) {
			param.put("engineId", dmpComputEngineRequest.getEngineId());
		}
		//引擎名称
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getEngineName())) {
			param.put("engineName", dmpComputEngineRequest.getEngineName());
		}
		//备注
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getRemark())) {
			param.put("remark", dmpComputEngineRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getDataStatus())) {
			param.put("dataStatus", dmpComputEngineRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpComputEngineRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpComputEngineRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpComputEngineRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpComputEngineRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpComputEngineRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpComputEngineRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpComputEngineRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpComputEngineRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpComputEngineRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpComputEngineRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpComputEngineRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpComputEngineRequest.getUpdateTimeEnd());
		}

		dmpComputEngineMapper.delete(param);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#deleteById(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineRequest, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseResponse softDelete(DmpComputEngineRequest dmpComputEngineRequest, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		Map<String, Object> param = new HashMap<>();
		//主键
		if (dmpComputEngineRequest.getEngineId() != null) {
			param.put("engineId", dmpComputEngineRequest.getEngineId());
		}
		//引擎名称
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getEngineName())) {
			param.put("engineName", dmpComputEngineRequest.getEngineName());
		}
		//备注
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getRemark())) {
			param.put("remark", dmpComputEngineRequest.getRemark());
		}
		//数据状态(0：删除，1，未删除)
		if (!StringUtils.isEmpty(dmpComputEngineRequest.getDataStatus())) {
			param.put("dataStatus", dmpComputEngineRequest.getDataStatus());
		}
		//创建用户ID
		if (dmpComputEngineRequest.getCreateUserId() != null) {
			param.put("createUserId", dmpComputEngineRequest.getCreateUserId());
		}
		//创建时间起
		if (dmpComputEngineRequest.getCreateTimeStart() != null) {
			param.put("createTimeStart", dmpComputEngineRequest.getCreateTimeStart());
		}
		//创建时间止
		if (dmpComputEngineRequest.getCreateTimeEnd() != null) {
			param.put("createTimeEnd", dmpComputEngineRequest.getCreateTimeEnd());
		}
		//修改用户ID
		if (dmpComputEngineRequest.getUpdateUserId() != null) {
			param.put("updateUserId", dmpComputEngineRequest.getUpdateUserId());
		}
		//修改时间起
		if (dmpComputEngineRequest.getUpdateTimeStart() != null) {
			param.put("updateTimeStart", dmpComputEngineRequest.getUpdateTimeStart());
		}
		//修改时间止
		if (dmpComputEngineRequest.getUpdateTimeEnd() != null) {
			param.put("updateTimeEnd", dmpComputEngineRequest.getUpdateTimeEnd());
		}

		dmpComputEngineMapper.softDelete(param);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ycxc.vmts.service.DmpComputEngineService#addBatch(com.ycxc.vmts.controller.bas.
	 * bean.DmpComputEngineVo, javax.servlet.http.HttpServletRequest)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BaseBeanResponse<DmpComputEngineBatch> addBatch(DmpComputEngineBatch dmpComputEngineBatch, HttpServletRequest httpRequest)
			throws Exception {
		BaseBeanResponse<DmpComputEngineBatch> baseBeanResponse = new BaseBeanResponse<>();

		List<DmpComputEngine> dmpComputEngines = dmpComputEngineBatch.getDmpComputEngines();

		if (CollectionUtils.isEmpty(dmpComputEngines)) {
			baseBeanResponse.setCode(StatuConstant.FAILURE_CODE);
			baseBeanResponse.setMessage("数据不能为空！");
			return baseBeanResponse;
		}

		dmpComputEngines = dmpComputEngines.stream().map(dmpComputEngine -> {
			/*$*setEngineIdCodeReplace*$*/
			// 创建人
			try {
				dmpComputEngine.setCreateUserId(getHttpRequestUserId(httpRequest));
			} catch (Exception e) {
				logger.info(e.getMessage());
				e.printStackTrace();
			}
			// 创建时间
			dmpComputEngine.setCreateTime(new Date());

			return dmpComputEngine;
		}).collect(Collectors.toList());

		dmpComputEngineMapper.insertBatch(dmpComputEngines);

		baseBeanResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseBeanResponse.setMessage("批量新增成功");
		baseBeanResponse.setData(dmpComputEngineBatch);

		return baseBeanResponse;
	}

	/**
	 *批量删除
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse deleteByIds(List<Integer> idList, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpComputEngineMapper.deleteByIds(idList);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("删除成功");

		return baseResponse;
	}

	/**
	 *批量软删除
	 */
	@Override
	@Transactional(rollbackFor=Exception.class)
	public BaseResponse softDeleteByIds(List<Integer> idList, HttpServletRequest httpRequest) throws Exception {
		BaseResponse baseResponse = new BaseResponse();

		dmpComputEngineMapper.softDeleteByIds(idList);

		baseResponse.setCode(StatuConstant.SUCCESS_CODE);
		baseResponse.setMessage("软删除成功");

		return baseResponse;
	}

}
